@@ -35,11 +35,9 @@ my $TESTS = {
     test_class => [qw(forking mod_redis reverse)],
   },
 
-  # This is flaky when run in GitHub workflows, but passes when run in Docker
-  # locally.  So marking it as flaky.
   proxy_reverse_config_redis_connect_policy_leastconns => {
     order => ++$order,
-    test_class => [qw(flaky forking mod_redis reverse)],
+    test_class => [qw(forking mod_redis reverse)],
   },
 
   proxy_reverse_config_redis_connect_policy_leastresponsetime => {
@@ -97,16 +95,10 @@ sub config_hash2array {
 sub get_redis_config {
   my $log_file = shift;
 
-  my $redis_server = '127.0.0.1';
-  if (defined($ENV{REDIS_HOST})) {
-    $redis_server = $ENV{REDIS_HOST};
-  }
-
   my $config = {
     RedisEngine => 'on',
     RedisLog => $log_file,
-    RedisServer => "$redis_server:6379",
-    RedisTimeouts => '2000 500',
+    RedisServer => '127.0.0.1:6379',
   };
 
   return $config;
@@ -399,6 +391,7 @@ sub proxy_reverse_config_redis_connect_policy_shuffle {
         DenyUser => $user,
       },
     },
+
   };
 
   my ($port, $config_user, $config_group) = config_write($config_file, $config);
@@ -445,8 +438,7 @@ EOC
     eval {
       for (my $i = 0; $i < $nbackends+1; $i++) {
         sleep(2);
-
-        my $client = ProFTPD::TestSuite::FTP->new('127.0.0.1', $port, 1, 3);
+        my $client = ProFTPD::TestSuite::FTP->new('127.0.0.1', $port, 1, 1);
         $client->login($user, $passwd);
         ftp_list($self, $client);
       }
@@ -648,13 +640,43 @@ EOC
 sub proxy_reverse_config_redis_connect_policy_leastconns {
   my $self = shift;
   my $tmpdir = $self->{tmpdir};
-  my $setup = test_setup($tmpdir, 'proxy');
+
+  my $config_file = "$tmpdir/proxy.conf";
+  my $pid_file = File::Spec->rel2abs("$tmpdir/proxy.pid");
+  my $scoreboard_file = File::Spec->rel2abs("$tmpdir/proxy.scoreboard");
+
+  my $log_file = test_get_logfile();
+
+  my $auth_user_file = File::Spec->rel2abs("$tmpdir/proxy.passwd");
+  my $auth_group_file = File::Spec->rel2abs("$tmpdir/proxy.group");
+
+  my $user = 'proftpd';
+  my $passwd = 'test';
+  my $group = 'ftpd';
+  my $home_dir = File::Spec->rel2abs($tmpdir);
+  my $uid = 500;
+  my $gid = 500;
+
+  # Make sure that, if we're running as root, that the home directory has
+  # permissions/privs set for the account we create
+  if ($< == 0) {
+    unless (chmod(0755, $home_dir)) {
+      die("Can't set perms on $home_dir to 0755: $!");
+    }
+
+    unless (chown($uid, $gid, $home_dir)) {
+      die("Can't set owner of $home_dir to $uid/$gid: $!");
+    }
+  }
+
+  auth_user_write($auth_user_file, $user, $passwd, $uid, $gid, $home_dir,
+    '/bin/bash');
+  auth_group_write($auth_group_file, $group, $gid, $user);
 
   my $vhost_port = ProFTPD::TestSuite::Utils::get_high_numbered_port();
   $vhost_port += 12;
 
-  my $proxy_config = get_reverse_proxy_config($tmpdir, $setup->{log_file},
-    $vhost_port);
+  my $proxy_config = get_reverse_proxy_config($tmpdir, $log_file, $vhost_port);
   $proxy_config->{ProxyReverseConnectPolicy} = 'LeastConns';
 
   # For now, we cheat and simply repeat the same vhost three times
@@ -663,17 +685,17 @@ sub proxy_reverse_config_redis_connect_policy_leastconns {
 
   my $timeout_idle = 10;
 
-  my $redis_config = get_redis_config($setup->{log_file});
+  my $redis_config = get_redis_config($log_file);
 
   my $config = {
-    PidFile => $setup->{pid_file},
-    ScoreboardFile => $setup->{scoreboard_file},
-    SystemLog => $setup->{log_file},
-    TraceLog => $setup->{log_file},
+    PidFile => $pid_file,
+    ScoreboardFile => $scoreboard_file,
+    SystemLog => $log_file,
+    TraceLog => $log_file,
     Trace => 'DEFAULT:10 event:0 lock:0 scoreboard:0 signal:0 proxy:20 proxy.reverse:20 proxy.reverse.redis:20 proxy.ftp.conn:20 proxy.ftp.ctrl:20 proxy.ftp.data:20 proxy.ftp.msg:20',
 
-    AuthUserFile => $setup->{auth_user_file},
-    AuthGroupFile => $setup->{auth_group_file},
+    AuthUserFile => $auth_user_file,
+    AuthGroupFile => $auth_group_file,
     SocketBindTight => 'on',
     TimeoutIdle => $timeout_idle,
 
@@ -688,26 +710,25 @@ sub proxy_reverse_config_redis_connect_policy_leastconns {
 
     Limit => {
       LOGIN => {
-        DenyUser => $setup->{user},
+        DenyUser => $user,
       },
     },
+
   };
 
-  my ($port, $config_user, $config_group) = config_write($setup->{config_file},
-    $config);
+  my ($port, $config_user, $config_group) = config_write($config_file, $config);
 
-  if (open(my $fh, ">> $setup->{config_file}")) {
+  if (open(my $fh, ">> $config_file")) {
     print $fh <<EOC;
 <VirtualHost 127.0.0.1>
   Port $vhost_port
   ServerName "Real Server"
 
-  AuthUserFile $setup->{auth_user_file}
-  AuthGroupFile $setup->{auth_group_file}
+  AuthUserFile $auth_user_file
+  AuthGroupFile $auth_group_file
   AuthOrder mod_auth_file.c
 
   AllowOverride off
-  RootLogin on
   TimeoutIdle $timeout_idle
 
   TransferLog none
@@ -715,11 +736,11 @@ sub proxy_reverse_config_redis_connect_policy_leastconns {
 </VirtualHost>
 EOC
     unless (close($fh)) {
-      die("Can't write $setup->{config_file}: $!");
+      die("Can't write $config_file: $!");
     }
 
   } else {
-    die("Can't open $setup->{config_file}: $!");
+    die("Can't open $config_file: $!");
   }
 
   # Open pipes, for use between the parent and child processes.  Specifically,
@@ -737,16 +758,14 @@ EOC
   defined(my $pid = fork()) or die("Can't fork: $!");
   if ($pid) {
     eval {
-      # Allow server to start up
-      sleep(2);
-
       for (my $i = 0; $i < $nbackends+1; $i++) {
-        my $client = ProFTPD::TestSuite::FTP->new('127.0.0.1', $port, 1, 2);
-        $client->login($setup->{user}, $setup->{passwd});
+        sleep(2);
+        my $client = ProFTPD::TestSuite::FTP->new('127.0.0.1', $port, 1, 1);
+        $client->login($user, $passwd);
         ftp_list($self, $client);
-        sleep(1);
       }
     };
+
     if ($@) {
       $ex = $@;
     }
@@ -755,7 +774,7 @@ EOC
     $wfh->flush();
 
   } else {
-    eval { server_wait($setup->{config_file}, $rfh, $timeout_idle + 2) };
+    eval { server_wait($config_file, $rfh, $timeout_idle + 2) };
     if ($@) {
       warn($@);
       exit 1;
@@ -765,10 +784,18 @@ EOC
   }
 
   # Stop server
-  server_stop($setup->{pid_file});
+  server_stop($pid_file);
+
   $self->assert_child_ok($pid);
 
-  test_cleanup($setup->{log_file}, $ex);
+  if ($ex) {
+    test_append_logfile($log_file, $ex);
+    unlink($log_file);
+
+    die($ex);
+  }
+
+  unlink($log_file);
 }
 
 sub proxy_reverse_config_redis_connect_policy_leastresponsetime {
